var html = "<section class=\"mt-10\"><h1 id=\"typescript\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#typescript\">#</a> TypeScript</h1><p>Solid is designed to be easy to use with TypeScript: its use of standard JSX makes code largely understood by TypeScript, and it provides sophisticated built-in types for its API. This guide covers some useful tips for working with TypeScript and typing your Solid code.</p><h2 id=\"configuring-typescript\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#configuring-typescript\">#</a> Configuring TypeScript</h2><p>The <a href=\"https://github.com/solidjs/templates/\">Solid starter templates</a> offer good starting points for <a href=\"https://github.com/solidjs/templates/blob/master/ts/tsconfig.json\"><code>tsconfig.json</code></a>.</p><p>Most importantly, to use TypeScript with the Solid JSX compiler, you need to configure TypeScript to leave JSX constructs alone via <a href=\"https://www.typescriptlang.org/tsconfig#jsx\"><code>&quot;jsx&quot;: &quot;preserve&quot;</code></a>, and tell TypeScript where the JSX types come from via <a href=\"https://www.typescriptlang.org/tsconfig#jsxImportSource\"><code>&quot;jsxImportSource&quot;: &quot;solid-js&quot;</code></a>. So, a minimal <code>tsconfig.json</code> would look like this:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#24292e\">{</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#005cc5\">&quot;compilerOptions&quot;</span><span style=\"color:#24292e\">: {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#005cc5\">&quot;jsx&quot;</span><span style=\"color:#24292e\">: </span><span style=\"color:#032f62\">&quot;preserve&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#005cc5\">&quot;jsxImportSource&quot;</span><span style=\"color:#24292e\">: </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span></code></pre><p>If your code base uses a mix of JSX types (e.g., some files are React while other files are Solid), you can set the default <code>jsxImportSource</code> in <code>tsconfig.json</code> for the majority of your code, and then <a href=\"https://www.typescriptlang.org/tsconfig#jsxImportSource\">override the <code>jsxImportSource</code> option</a> in specific <code>.tsx</code> files using the following pragma:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">/** </span><span style=\"color:#d73a49\">@jsxImportSource</span><span style=\"color:#6a737d\"> solid-js */</span></span>\n<span class=\"line\"></span></code></pre><p>or</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">/** </span><span style=\"color:#d73a49\">@jsxImportSource</span><span style=\"color:#6a737d\"> react */</span></span>\n<span class=\"line\"></span></code></pre><h2 id=\"api-types\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#api-types\">#</a> API Types</h2><p>Solid is written in TypeScript, so everything is typed out of the box. The <a href=\"https://www.solidjs.com/docs/latest/api\">API documentation</a> details the types for all API calls, as well as several helpful type definitions to make it easier to refer to Solid notions when you need to specify explicit types. Here, we explore the resulting types when using a few core primitives.</p><h3 id=\"signals\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#signals\">#</a> Signals</h3><p><code>createSignal&lt;T&gt;</code> is parameterized by the type <code>T</code> of the object stored in the signal. For example:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">count</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setCount</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">number</span><span style=\"color:#24292e\">&gt;();</span></span>\n<span class=\"line\"></span></code></pre><p>The first <code>createSignal</code> has return type <code>Signal&lt;number&gt;</code>, corresponding to the type we passed to it. This is a tuple of the getter and setter, which each have a generic type <code>Accessor&lt;T&gt;</code> and <code>Setter&lt;T&gt;</code>.</p><p>In this case, the signal getter <code>count</code> has type <code>Accessor&lt;number | undefined&gt;</code>. <code>Accessor&lt;T&gt;</code> is a type definition provided by Solid, in this case equivalent to <code>() =&gt; number | undefined</code>. The <code>| undefined</code> gets added in this example because we did not provide a default value to <code>createSignal</code>, so the signal value indeed starts out as <code>undefined</code>.</p><p>The signal setter <code>setCount</code> has type <code>Setter&lt;number&gt;</code>, which is a more complicated type definition corresponding roughly to <code>(value?: number | ((prev?: number) =&gt; number)) =&gt; number</code>, representing the two possibilities for the passed argument: you can call <code>setCount</code> with a simple <code>number</code>, or a function taking the previous value (if there was one) and returning a number.</p><h4 id=\"defaults\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#defaults\">#</a> Defaults</h4><p>We can avoid having to explicitly provide the type of the signal when calling <code>createSignal</code>, and avoid the <code>| undefined</code> part of the type, by providing a default value to <code>createSignal</code>:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">count</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setCount</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">name</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setName</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&#39;&#39;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span></code></pre><p>In this case, TypeScript infers that the signal types are <code>number</code> and <code>string</code> respectively. Thus, for example, <code>count</code> obtains type <code>Accessor&lt;number&gt;</code> and <code>name</code> obtains type <code>Accessor&lt;string&gt;</code> (without <code>| undefined</code>).</p><h3 id=\"context\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#context\">#</a> Context</h3><p>Similar to signals, <a href=\"https://www.solidjs.com/docs/latest/api#createcontext\"><code>createContext&lt;T&gt;</code></a> is parameterized by the type <code>T</code> of the context value. We can provide this type explicitly:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">type</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Data</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> {</span><span style=\"color:#e36209\">count</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">number</span><span style=\"color:#24292e\">, </span><span style=\"color:#e36209\">name</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">dataContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createContext</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">Data</span><span style=\"color:#24292e\">&gt;();</span></span>\n<span class=\"line\"></span></code></pre><p>In this case, <code>dataContext</code> has type <code>Context&lt;Data | undefined&gt;</code>, causing <code>useContext(dataContext)</code> to have matching return type <code>Data | undefined</code>. The reason for <code>| undefined</code> is that the context might not be provided in the ancestors of the current component, in which case <code>useContext</code> returns <code>undefined</code>.</p><p>If we instead provide a default value to <code>createContext</code>, we avoid the <code>| undefined</code> part of the type, and often avoid having to explicitly specify the type of the <code>createContext</code> as well:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">dataContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createContext</span><span style=\"color:#24292e\">({count: </span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">, name: </span><span style=\"color:#032f62\">&#39;&#39;</span><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span></code></pre><p>In this case, TypeScript infers that <code>dataContext</code> has type <code>Context&lt;{count: number, name: string}&gt;</code>, which is equivalent to <code>Context&lt;Data&gt;</code> (without <code>| undefined</code>).</p><p>Another common pattern is to define a factory function that produces the value for a context. Then we can grab the return type of that function using TypeScript's [<code>ReturnType</code>](<a href=\"https://www.typescriptlang.org/docs/handbook/utility-types\">https://www.typescriptlang.org/docs/handbook/utility-types</a>. html#returntypetype) type helper, and use that to type the context.</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">makeCountNameContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> (</span><span style=\"color:#e36209\">initialCount</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">, </span><span style=\"color:#e36209\">initialName</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&#39;&#39;</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">count</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setCount</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(initialCount);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">name</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setName</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(initialName);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> [{count, name}, {setCount, setName}] </span><span style=\"color:#d73a49\">as</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6a737d\">// `as const` forces tuple type inference</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">type</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">CountNameContextType</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">ReturnType</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#d73a49\">typeof</span><span style=\"color:#24292e\"> makeCountNameContext&gt;;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">CountNameContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createContext</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">CountNameContextType</span><span style=\"color:#24292e\">&gt;();</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">useCountNameContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">useContext</span><span style=\"color:#24292e\">(CountNameContext);</span></span>\n<span class=\"line\"></span></code></pre><p>In this example, <code>CountNameContextType</code> corresponds to the return value of <code>makeCountNameContext</code>:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#24292e\">[</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  {readonly count: Accessor</span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">number</span><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\">, readonly name: Accessor</span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">string</span><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\">},</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  {readonly setCount: Setter</span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">number</span><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\">, readonly setName: Setter</span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">string</span><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">]</span></span>\n<span class=\"line\"></span></code></pre><p>and <code>useCountNameContext</code> has type <code>() =&gt; CountNameContextType | undefined</code>.</p><p>If you want to avoid the <code>undefined</code> possibility, you could assert that the context is always provided when used:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">useCountNameContext</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">useContext</span><span style=\"color:#24292e\">(CountNameContext)</span><span style=\"color:#d73a49\">!</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span></code></pre><p>This is a dangerous assumption; it would be safer to actually provide a default argument to <code>createContext</code> so that the context is always defined.</p><h2 id=\"component-types\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#component-types\">#</a> Component Types</h2><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">type</span><span style=\"color:#24292e\"> {PropsWithChildren, Component} </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&#39;solid-js&#39;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">type</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">PropsWithChildren</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">P</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> {}&gt; </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">P</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">&amp;</span><span style=\"color:#24292e\"> { </span><span style=\"color:#e36209\">children</span><span style=\"color:#d73a49\">?:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\">.</span><span style=\"color:#e36209\">Element</span><span style=\"color:#24292e\"> };</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">type</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Component</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">P</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> {}&gt; </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> (</span><span style=\"color:#e36209\">props</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">PropsWithChildren</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">P</span><span style=\"color:#24292e\">&gt;) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\">.</span><span style=\"color:#e36209\">Element</span></span>\n<span class=\"line\"></span></code></pre><p>Solid provides the <code>Component&lt;P&gt;</code> type to represent a component function, where the <code>props</code> argument has type <code>P</code>. Here <code>P</code> should be an object type, and doesn't need to explicitly mention the <code>children</code> property; <code>{ children?: JSX.Element }</code> is automatically added to the type (via the <code>PropsWithChildren&lt;P&gt;</code> type definition). For example:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">Counter</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Component</span><span style=\"color:#24292e\">&lt;{</span><span style=\"color:#e36209\">initialValue</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">number</span><span style=\"color:#24292e\">}&gt; </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> (</span><span style=\"color:#e36209\">props</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  [count, setCount] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(props.initialValue);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">button onClick</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">setCount</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">c</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">c</span><span style=\"color:#24292e\">+</span><span style=\"color:#005cc5\">1</span><span style=\"color:#24292e\">)}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      {</span><span style=\"color:#e36209\">count</span><span style=\"color:#24292e\">()}</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      {</span><span style=\"color:#e36209\">props</span><span style=\"color:#24292e\">.</span><span style=\"color:#e36209\">children</span><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">button</span><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"></span></code></pre><p>This code automatically types <code>props</code> to <code>{initialValue: number, children?: JSX.Element}</code> and forces a return value of <code>JSX.Element</code> (which can be a DOM node, an array of <code>JSX.Element</code>s, a function returning a <code>JSX.Element</code>, a boolean, or anything else the renderer can handle).</p><p>The namespace <code>JSX</code> offers a suite of useful types for working with HTML DOM in particular. See the <a href=\"https://github.com/ryansolid/dom-expressions/blob/main/packages/dom-expressions/src/jsx.d.ts\">definition of JSX in dom-expressions</a> for all the types provided.</p><h2 id=\"event-handlers\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#event-handlers\">#</a> Event Handlers</h2><p>One useful helper type provided by the <code>JSX</code> namespace is <code>JSX.EventHandler&lt;T&gt;</code>, which represents a single-argument event handler for a DOM element type <code>T</code>. You can use this to type any event handlers you define outside JSX. For example:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">onInput</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\">.</span><span style=\"color:#e36209\">EventHandler</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">HTMLInputElement</span><span style=\"color:#24292e\">&gt; </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> (</span><span style=\"color:#e36209\">event</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  console.</span><span style=\"color:#6f42c1\">log</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&#39;input changed to&#39;</span><span style=\"color:#24292e\">, event.currentTarget.value);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">input onInput</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{onInput}</span><span style=\"color:#d73a49\">/&gt;</span></span>\n<span class=\"line\"></span></code></pre><p>Handlers defined inline within <a href=\"https://www.solidjs.com/docs/latest/api#on___\"><code>on___</code> JSX attributes</a> (with built-in event types) are automatically typed as the appropriate <code>JSX.EventHandler</code>:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">input onInput</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{(event) =&gt; {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  console.log(</span><span style=\"color:#032f62\">&#39;input changed to&#39;</span><span style=\"color:#24292e\">, event.currentTarget.value);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}}</span><span style=\"color:#d73a49\">/&gt;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span></code></pre><p>Note that <code>JSX.EventHandler&lt;T&gt;</code> constrains the event's <a href=\"https://developer.mozilla.org/en-US/docs/Web/API/Event/currentTarget\"><code>currentTarget</code> attribute</a> to be of type <code>T</code> (in the example, <code>event.currentTarget</code> is typed as <code>HTMLInputEvent</code>, so has attribute <code>value</code>). However, the event's <a href=\"https://developer.mozilla.org/en-US/docs/Web/API/Event/target\"><code>target</code> attribute</a> could be any <code>DOMElement</code>. This is the nature of DOM events: <code>currentTarget</code> is the element that the event handler was attached to, so has a known type, whereas <code>target</code> is whatever the user interacted with that caused the event to bubble to or get captured by the event handler, which can be any DOM element.</p><h2 id=\"custom-events%2C-properties%2C-attributes%2C-and-directives\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#custom-events%2C-properties%2C-attributes%2C-and-directives\">#</a> Custom Events, Properties, Attributes, and Directives</h2><p>If you use custom event handlers via Solid's <a href=\"https://www.solidjs.com/docs/latest/api#on%3A___%2Foncapture%3A___\"><code>on:___</code>/<code>oncapture:___</code> attributes</a>, you should define corresponding types for the resulting <code>Event</code> objects, by overriding the <code>CustomEvents</code> and <code>CustomCaptureEvents</code> interfaces within module <code>&quot;solid-js&quot;</code>'s <code>JSX</code> namespace, like so:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">class</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">NameEvent</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">extends</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">CustomEvent</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#e36209\">type</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&#39;Name&#39;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#e36209\">detail</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> {</span><span style=\"color:#e36209\">name</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">constructor</span><span style=\"color:#24292e\">(</span><span style=\"color:#e36209\">name</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">) {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#005cc5\">super</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&#39;Name&#39;</span><span style=\"color:#24292e\">, {detail: {name}});</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">declare</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">module</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">namespace</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">interface</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">CustomEvents</span><span style=\"color:#24292e\"> { </span><span style=\"color:#6a737d\">// on:Name</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#032f62\">&quot;Name&quot;</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">NameEvent</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">interface</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">CustomCaptureEvents</span><span style=\"color:#24292e\"> { </span><span style=\"color:#6a737d\">// oncapture:Name</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#032f62\">&quot;Name&quot;</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">NameEvent</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">div </span><span style=\"color:#e36209\">on</span><span style=\"color:#24292e\">:Name</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{(event) =&gt; console.log(</span><span style=\"color:#032f62\">&#39;name is&#39;</span><span style=\"color:#24292e\">, event.detail.name)}</span><span style=\"color:#d73a49\">/&gt;</span></span>\n<span class=\"line\"></span></code></pre><p>If you use forced properties via Solid's <a href=\"https://www.solidjs.com/docs/latest/api#prop%3A___\"><code>prop:___</code> attributes</a>, or custom attributes via Solid's <a href=\"https://www.solidjs.com/docs/latest/api#attr%3A___\"><code>attr:___</code> attributes</a>, you can define their types in the <code>ExplicitProperties</code> and <code>ExplicitAttributes</code> interfaces, respectively:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">declare</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">module</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">namespace</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">interface</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">ExplicitProperties</span><span style=\"color:#24292e\"> { </span><span style=\"color:#6a737d\">// prop:___</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">count</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">number</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">name</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">interface</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">ExplicitAttributes</span><span style=\"color:#24292e\"> { </span><span style=\"color:#6a737d\">// attr:___</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">count</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">number</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">name</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">Input </span><span style=\"color:#e36209\">prop</span><span style=\"color:#24292e\">:name</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">name</span><span style=\"color:#24292e\">()} </span><span style=\"color:#e36209\">prop</span><span style=\"color:#24292e\">:count</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">count</span><span style=\"color:#24292e\">()}</span><span style=\"color:#d73a49\">/&gt;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">my</span><span style=\"color:#d73a49\">-</span><span style=\"color:#24292e\">web</span><span style=\"color:#d73a49\">-</span><span style=\"color:#24292e\">component </span><span style=\"color:#e36209\">attr</span><span style=\"color:#24292e\">:name</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">name</span><span style=\"color:#24292e\">()} </span><span style=\"color:#e36209\">attr</span><span style=\"color:#24292e\">:count</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">count</span><span style=\"color:#24292e\">()}</span><span style=\"color:#d73a49\">/&gt;</span></span>\n<span class=\"line\"></span></code></pre><p>If you define custom directives for Solid's <a href=\"https://www.solidjs.com/docs/latest/api#use%3A___\"><code>use:___</code> attributes</a>, you can type them in the <code>Directives</code> interface, like so:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">function</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">model</span><span style=\"color:#24292e\">(</span><span style=\"color:#e36209\">element</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">HTMLInputElement</span><span style=\"color:#24292e\">, </span><span style=\"color:#e36209\">value</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Accessor</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">Signal</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">&gt;&gt;) {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">field</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setField</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">value</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createRenderEffect</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> (element.value </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">field</span><span style=\"color:#24292e\">()));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  element.</span><span style=\"color:#6f42c1\">addEventListener</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;input&quot;</span><span style=\"color:#24292e\">, (</span><span style=\"color:#e36209\">e</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">setField</span><span style=\"color:#24292e\">(e.target.value));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">declare</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">module</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">namespace</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">JSX</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">interface</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Directives</span><span style=\"color:#24292e\"> {  </span><span style=\"color:#6a737d\">// use:model</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">model</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Signal</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">&gt;;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> [name, setName] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&#39;&#39;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">input type</span><span style=\"color:#d73a49\">=</span><span style=\"color:#032f62\">&quot;text&quot;</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">use</span><span style=\"color:#24292e\">:model</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{[name, setName]} /&gt;;</span></span>\n<span class=\"line\"></span></code></pre><h2 id=\"ref\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#ref\">#</a> ref</h2><p>Here is the typical pattern for using <code>ref</code> with TypeScript:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> divRef</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">HTMLDivElement</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> buttonRef</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">HTMLButtonElement</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">div ref</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{divRef!}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">button ref</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{buttonRef!}</span><span style=\"color:#d73a49\">&gt;...&lt;/</span><span style=\"color:#24292e\">button</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">div</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span></code></pre><p>Note the explicit types on the declarations, and the <a href=\"https://www.typescriptlang.org/docs/handbook/2/everyday-types.html#non-null-assertion-operator-postfix-\">non-null assertions (<code>!</code>)</a> when using the <code>ref</code>. (Without the non-null assertion, TypeScript thinks that the <code>ref</code> attribute is being given the ref variable, when in fact it's setting the ref variable. This is a workaround until TypeScript can understand assignments to variables through JSX attributes.)</p><p>Alternatively, you can declare the ref variables as non-null from the get go:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> divRef</span><span style=\"color:#d73a49\">!:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">HTMLDivElement</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> buttonRef</span><span style=\"color:#d73a49\">!:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">HTMLButtonElement</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span></code></pre><p>But be careful: these ref variables won't actually be set (and non-null) until after the rendering phase. You can safely use them in a <a href=\"https://www.solidjs.com/docs/latest/api#createeffect\"><code>createEffect</code></a>, for example, but not in the body of the component function.</p><h2 id=\"control-flow-narrowing\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#control-flow-narrowing\">#</a> Control Flow Narrowing</h2><p>A common pattern is to use <a href=\"https://www.solidjs.com/docs/latest/api#%3Cshow%3E\"><code>&lt;Show&gt;</code></a> to display data only when that data is defined:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">name</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setName</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">string</span><span style=\"color:#24292e\">&gt;();</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">Show when</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">name</span><span style=\"color:#24292e\">()}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    Hello {</span><span style=\"color:#e36209\">name</span><span style=\"color:#24292e\">().</span><span style=\"color:#e36209\">replace</span><span style=\"color:#24292e\">(/\\</span><span style=\"color:#e36209\">s</span><span style=\"color:#24292e\">+/</span><span style=\"color:#e36209\">g</span><span style=\"color:#24292e\">, &#39;\\</span><span style=\"color:#e36209\">xa0</span><span style=\"color:#24292e\">&#39;)}</span><span style=\"color:#d73a49\">!</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">Show</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span></code></pre><p>In this case, TypeScript can't determine that the two calls to <code>name()</code> will return the same value, and that the second call will happen only if the first call returned a truthy value. Thus it will complain that <code>name()</code> might be <code>undefined</code> when trying to call <code>.replace()</code>.</p><p>Here are two workarounds for this issue:</p><ol> <li> <p>You can manually assert that <code>name()</code> will be non-null in the second call using TypeScript's <a href=\"https://www.typescriptlang.org/docs/handbook/2/everyday-types.html#non-null-assertion-operator-postfix-\">non-null assertion operator <code>!</code></a>:</p> <pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">Show when</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">name</span><span style=\"color:#24292e\">()}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    Hello {</span><span style=\"color:#e36209\">name</span><span style=\"color:#24292e\">().</span><span style=\"color:#e36209\">replace</span><span style=\"color:#24292e\">(/\\</span><span style=\"color:#e36209\">s</span><span style=\"color:#24292e\">+/</span><span style=\"color:#e36209\">g</span><span style=\"color:#24292e\">, &#39;\\</span><span style=\"color:#e36209\">xa0</span><span style=\"color:#24292e\">&#39;)}</span><span style=\"color:#d73a49\">!</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">Show</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span></code></pre> </li> <li> <p>You can use the callback form of <code>&lt;Show&gt;</code>, which passes in the value of the <code>when</code> prop when it is truthy:</p> <pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">Show when</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{</span><span style=\"color:#6f42c1\">name</span><span style=\"color:#24292e\">()}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    {(</span><span style=\"color:#e36209\">n</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      &lt;&gt;</span><span style=\"color:#e36209\">Hello</span><span style=\"color:#24292e\"> {</span><span style=\"color:#e36209\">n</span><span style=\"color:#24292e\">.</span><span style=\"color:#e36209\">replace</span><span style=\"color:#24292e\">(/\\</span><span style=\"color:#e36209\">s</span><span style=\"color:#24292e\">+/</span><span style=\"color:#e36209\">g</span><span style=\"color:#24292e\">, &#39;\\</span><span style=\"color:#e36209\">xa0</span><span style=\"color:#24292e\">&#39;)}!&lt;/&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">Show</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span></code></pre> <p>In this case, the typing of the <code>Show</code> component is clever enough to tell TypeScript that <code>n</code> is truthy, so can't be <code>undefined</code> (or <code>null</code> or <code>false</code>).</p> <p>Note, however, that this form of <code>&lt;Show&gt;</code> forces the children to render from scratch every time <code>name()</code> changes, instead of just rendering from scratch when <code>name()</code> changes from a falsey to a truthy value. This prevents the children from the benefits of fine-grained reactivity (re-using unchanged parts and updating just what changed).</p> </li> </ol></section>";
var attributes = {
	title: "TypeScript",
	description: "Tips for using Solid with TypeScript",
	sort: 3
};
var sections = [
	{
		slug: "typescript",
		title: "TypeScript",
		level: 1,
		children: [
			{
				slug: "configuring-typescript",
				title: "Configuring TypeScript",
				level: 2,
				children: [
				]
			},
			{
				slug: "api-types",
				title: "API Types",
				level: 2,
				children: [
					{
						slug: "signals",
						title: "Signals",
						level: 3,
						children: [
							{
								slug: "defaults",
								title: "Defaults",
								level: 4,
								children: [
								]
							}
						]
					},
					{
						slug: "context",
						title: "Context",
						level: 3,
						children: [
						]
					}
				]
			},
			{
				slug: "component-types",
				title: "Component Types",
				level: 2,
				children: [
				]
			},
			{
				slug: "event-handlers",
				title: "Event Handlers",
				level: 2,
				children: [
				]
			},
			{
				slug: "custom-events%2C-properties%2C-attributes%2C-and-directives",
				title: "Custom Events, Properties, Attributes, and Directives",
				level: 2,
				children: [
				]
			},
			{
				slug: "ref",
				title: "ref",
				level: 2,
				children: [
				]
			},
			{
				slug: "control-flow-narrowing",
				title: "Control Flow Narrowing",
				level: 2,
				children: [
				]
			}
		]
	}
];
var typescript = {
	html: html,
	attributes: attributes,
	sections: sections
};

export { attributes, typescript as default, html, sections };
